@@ -1,12 +1,13 @@
-#' Calculates all possible subsets and selects those where each included covariate is significant. 
+#' Calculates all possible subsets and selects those where each included covariate is significant. Decode with decode.
 #'
 #' 
 #'
 #' @param y The dependent variable.
 #' @param x  The covariates.
 #' @param p0 Cut-off p-value for significance.
-#' @param q The number of covarites from which to choose 
+#' @param q The number of covariates from which to choose. 
 #' @param ind   The indices of subset of covariates for which all subsets are to be considered.
+#' @param kmx If kmx > 0 the maximum number of included covariates for the given cut-off P-value.
 #' @param select Logical. If TRUE remove all subsets of chosen sets.
 #' @param inr Logical If TRUE include intercept.
 #' @param xinr Logical If TRUE intercept already included.
@@ -15,7 +16,7 @@
 #' @examples 
 #' data(redwine)
 #' a<-fmch(redwine[,12],redwine[,1:11])
-fmch<-function(y,x,p0=0.01,q=-1,ind=0,sel=T,inr=T,xinr=F){
+fmch<-function(y,x,p0=0.01,q=-1,ind=0,kmx=0,sel=T,inr=T,xinr=F){
 	n<-length(y)
 	x<-matrix(x,nrow=n)
 	k<-length(x)/n
@@ -35,7 +36,8 @@ fmch<-function(y,x,p0=0.01,q=-1,ind=0,sel=T,inr=T,xinr=F){
 	}
 	else{ind<-1:k}
 	if(q==-1){q<-k}
-	ss<-double(2^k+2)
+	kmxx<-2^k
+	if(xinr){kmxx<-2^(k-1)}
 	tmp<-.Fortran(
 		"lmmdch",
 		as.double(y),
@@ -49,27 +51,26 @@ fmch<-function(y,x,p0=0.01,q=-1,ind=0,sel=T,inr=T,xinr=F){
 		double(k),
 		double(k),
 		double(k),
-		double(k^2),
+		double(k**2),
 		integer(k),
 		as.logical(xinr),
-		double(2^k+2),
-		integer(2^(k+2)),
-		as.double(ss),
+		double(2**k+2),
+		integer(2**(k+1)),
+		double(2**k+2),
 		as.double(p0),
 		as.integer(q)
 	)
 	ss<-tmp[[17]]
-	inv<-(1:(2^k+2))[ss>0]
+	inv<-(1:(2**k+2))[ss>0]
 	ss<-ss[inv]
+	rs<-rank(ss)
 	nv<-tmp[[16]]
-	dim(nv)<-c(2^(k+1),2)
+	dim(nv)<-c(2**k,2)
 	nv<-matrix(nv,ncol=2)
 	nv<-nv[inv,]
 	nv<-matrix(nv,ncol=2)
 	llv<-length(inv)
 	if(llv>0){
-		nv<-tmp[[16]]
-		nv<-matrix(nv,ncol=2)
 		ind<-rank(ss,ties.method="first")
 		inv<-1:llv
 		inv[ind]<-inv
@@ -84,15 +85,21 @@ fmch<-function(y,x,p0=0.01,q=-1,ind=0,sel=T,inr=T,xinr=F){
 		nvv<-cbind(nv1,nv2,ss)
 		nvv<-matrix(nvv,ncol=3)
 		if(sel&(llv>1)){
-			ind<-rank(nv2,ties.method="first")	
+			ind<-rank(-nv2,ties.method="first")	
 			inv<-1:llv		
 			inv[ind]<-inv
-			indd<-fselect(nvv,k)[[1]]
-			nv1<-nv1[indd]
-			nv2<-nv2[indd]
-			ss<-ss[indd]
-			nvv<-cbind(nv1,nv2,ss)
-			nvv<-matrix(nvv,ncol=3)
+			nvv<-nvv[inv,]
+			kk<-k
+			if(xinr){kk<-k-1}
+			indd<-fselect(nvv,kk)[[1]]
+			li<-length(indd)
+			if(li==1){nvv<-nvv[li,]}
+			else{li<-1:li
+				nvv<-nvv[indd,]
+				rl<-rank(nvv[,3])
+				li[rl]<-li
+				nvv<-nvv[li,]
+			}
 		}
 		if(llv==1){nvv<-cbind(nv1[1],nv2[1],ss[1])
 			nvv<-matrix(nvv,nrow=1)
